
#include "hardopl.h"
#include "c86ctl/c86ctl.h"

namespace HARDOPL {

Chip::Chip() {
	hardActive = false;
	opl3Active = 0;
	handle = 0;
	chipBase = 0;
	gimic = 0;
	realChip = 0;
}

Chip::~Chip() {
	Shutdown();
}

void Chip::Init() {
	Shutdown();

	handle = ::LoadLibrary( "c86ctl.dll" );
	if ( handle == NULL ) {
		return;
	}

	typedef HRESULT (WINAPI *TCreateInstance)( REFIID, LPVOID* );
	TCreateInstance pCI;
	pCI = (TCreateInstance)::GetProcAddress( handle, "CreateInstance" );
	if ( !pCI ) {
		Shutdown();
		return;
	}

	(*pCI)( IID_IRealChipBase, (void**)&chipBase );
	chipBase->initialize();

	int count = chipBase->getNumberOfChip();
	if ( count != 0 ) {
		for ( int i=0; i < count; i++ ) {
			chipBase->getChipInterface( i, IID_IGimic, (void**)&gimic );
			chipBase->getChipInterface( i, IID_IRealChip, (void**)&realChip );

			Devinfo info;
			gimic->getModuleInfo( &info );
			if ( strncmp( info.Devname, "GMC-OPL3", 8 ) == 0 ) {
				hardActive = true;
				break;
			}
		}
	}
}

void Chip::Shutdown() {
	hardActive = false;
	if ( chipBase ) {
		chipBase->deinitialize();
	}
	if ( handle ) {
		::FreeLibrary( handle );
	}
}

Bit32u Chip::WriteAddr( Bit32u port, Bit8u val ) {
	switch ( port & 3 ) {
	case 0:
		return val;
	case 2:
		if ( opl3Active || (val == 0x05) )
			return 0x100 | val;
		else 
			return val;
	}
	return 0;
}

void Chip::WriteReg( Bit32u reg, Bit8u val ) {
	if ( reg == 0x105 ) {
		opl3Active = ( val & 1 ) ? true : false;
	}

	if ( hardActive ) {
		realChip->out( reg, val );
	}
}

Bit32u Handler::WriteAddr( Bit32u port, Bit8u val ) {
	return chip.WriteAddr( port, val );

}
void Handler::WriteReg( Bit32u addr, Bit8u val ) {
	chip.WriteReg( addr, val );
}

void Handler::Generate( MixerChannel* chan, Bitu samples ) {
	Bit32s buffer[ 512 * 2 ];
	if ( GCC_UNLIKELY(samples > 512) )
		samples = 512;
	memset(buffer, 0, sizeof(Bit32s) * samples);
	chan->AddSamples_m32( samples, buffer );
}

void Handler::Init( Bitu rate ) {
	chip.Init();
}

};		//Namespace HARDOPL
